@@ -10,22 +10,22 @@
 
 """Injector - Python dependency injection framework, inspired by Guice
 
-See https://github.com/alecthomas/injector for documentation.
-
 :copyright: (c) 2012 by Alec Thomas
 :license: BSD
 """
 
-import itertools
 import functools
 import inspect
+import itertools
 import logging
 import sys
-import types
 import threading
+import types
+import warnings
 from abc import ABCMeta, abstractmethod
 from collections import namedtuple
 
+
 try:
     NullHandler = logging.NullHandler
 except AttributeError:
@@ -34,7 +34,7 @@ except AttributeError:
             pass
 
 __author__ = 'Alec Thomas <alec@swapoff.org>'
-__version__ = '0.7.8'
+__version__ = '0.8.0'
 __version_tag__ = ''
 
 log = logging.getLogger('injector')
@@ -83,11 +83,8 @@ class UnsatisfiedRequirement(Error):
 
     def __str__(self):
         on = '%s has an ' % _describe(self.args[0]) if self.args[0] else ''
-        return '%sunsatisfied requirement on %s%s' % (
-            on,
-            self.args[1].annotation + '='if self.args[1].annotation else '',
-            _describe(self.args[1].interface),
-            )
+        return '%sunsatisfied requirement on %s' % (
+            on, _describe(self.args[1].interface),)
 
 
 class CallError(Error):
@@ -146,7 +143,26 @@ class ClassProvider(Provider):
 
 
 class CallableProvider(Provider):
-    """Provides something using a callable."""
+    """Provides something using a callable.
+
+    The callable is called every time new value is requested from the provider.
+
+    ::
+
+        >>> key = Key('key')
+        >>> def factory():
+        ...     print('providing')
+        ...     return []
+        ...
+        >>> def configure(binder):
+        ...     binder.bind(key, to=CallableProvider(factory))
+        ...
+        >>> injector = Injector(configure)
+        >>> injector.get(key) is injector.get(key)
+        providing
+        providing
+        False
+        """
 
     def __init__(self, callable):
         self._callable = callable
@@ -156,7 +172,21 @@ class CallableProvider(Provider):
 
 
 class InstanceProvider(Provider):
-    """Provide a specific instance."""
+    """Provide a specific instance.
+
+    ::
+
+        >>> my_list = Key('my_list')
+        >>> def configure(binder):
+        ...     binder.bind(my_list, to=InstanceProvider([]))
+        ...
+        >>> injector = Injector(configure)
+        >>> injector.get(my_list) is injector.get(my_list)
+        True
+        >>> injector.get(my_list).append('x')
+        >>> injector.get(my_list)
+        ['x']
+    """
 
     def __init__(self, instance):
         self._instance = instance
@@ -201,38 +231,38 @@ class MapBindProvider(ListOfProviders):
 class BindingKey(tuple):
     """A key mapping to a Binding."""
 
-    def __new__(cls, what, annotation):
+    def __new__(cls, what):
         if isinstance(what, list):
             if len(what) != 1:
                 raise Error('list bindings must have a single interface '
                             'element')
-            what = (list, BindingKey(what[0], None))
+            what = (list, BindingKey(what[0]))
         elif isinstance(what, dict):
             if len(what) != 1:
                 raise Error('dictionary bindings must have a single interface '
                             'key and value')
-            what = (dict, BindingKey(list(what.items())[0], None))
-        return tuple.__new__(cls, (what, annotation))
+            what = (dict, BindingKey(list(what.items())[0]))
+        return tuple.__new__(cls, (what,))
 
     @property
     def interface(self):
         return self[0]
 
-    @property
-    def annotation(self):
-        return self[1]
 
-
-_BindingBase = namedtuple('_BindingBase', 'interface annotation provider scope')
+_BindingBase = namedtuple('_BindingBase', 'interface provider scope')
 
 
 @private
 class Binding(_BindingBase):
-    """A binding from an (interface, annotation) to a provider in a scope."""
+    """A binding from an (interface,) to a provider in a scope."""
 
 
 class Binder(object):
-    """Bind interfaces to implementations."""
+    """Bind interfaces to implementations.
+
+    .. note:: This class is instantiated internally for you and there's no need
+        to instantiate it on your own.
+    """
 
     @private
     def __init__(self, injector, auto_bind=True, parent=None):
@@ -247,20 +277,19 @@ class Binder(object):
         self._bindings = {}
         self.parent = parent
 
-    def bind(self, interface, to=None, annotation=None, scope=None):
+    def bind(self, interface, to=None, scope=None):
         """Bind an interface to an implementation.
 
         :param interface: Interface or :func:`Key` to bind.
         :param to: Instance or class to bind to, or an explicit
              :class:`Provider` subclass.
-        :param annotation: Optional global annotation of interface.
         :param scope: Optional :class:`Scope` in which to bind.
         """
         if type(interface) is type and issubclass(interface, (BaseMappingKey, BaseSequenceKey)):
-            return self.multibind(interface, to, annotation=annotation, scope=scope)
-        key = BindingKey(interface, annotation)
+            return self.multibind(interface, to, scope=scope)
+        key = BindingKey(interface)
         self._bindings[key] = \
-            self.create_binding(interface, to, annotation, scope)
+            self.create_binding(interface, to, scope)
 
     def bind_scope(self, scope):
         """Bind a Scope.
@@ -269,7 +298,7 @@ class Binder(object):
         """
         self.bind(scope, to=scope(self.injector))
 
-    def multibind(self, interface, to, annotation=None, scope=None):
+    def multibind(self, interface, to, scope=None):
         """Creates or extends a multi-binding.
 
         A multi-binding maps from a key to a sequence, where each element in
@@ -278,17 +307,17 @@ class Binder(object):
         :param interface: :func:`MappingKey` or :func:`SequenceKey` to bind to.
         :param to: Instance, class to bind to, or an explicit :class:`Provider`
                 subclass. Must provide a sequence.
-        :param annotation: Optional global annotation of interface.
         :param scope: Optional Scope in which to bind.
         """
-        key = BindingKey(interface, annotation)
+        key = BindingKey(interface)
         if key not in self._bindings:
-            if isinstance(interface, dict) or isinstance(interface, type) and issubclass(interface, dict):
+            if (
+                    isinstance(interface, dict) or
+                    isinstance(interface, type) and issubclass(interface, dict)):
                 provider = MapBindProvider()
             else:
                 provider = MultiBindProvider()
-            binding = self.create_binding(
-                interface, provider, annotation, scope)
+            binding = self.create_binding(interface, provider, scope)
             self._bindings[key] = binding
         else:
             binding = self._bindings[key]
@@ -299,8 +328,43 @@ class Binder(object):
     def install(self, module):
         """Install a module into this binder.
 
-        :param module: A Module instance, Module subclass, or a function.
+        In this context the module is one of the following:
+
+        * function taking the :class:`Binder` as it's only parameter
+
+          ::
+
+            def configure(binder):
+                bind(str, to='s')
+
+            binder.install(configure)
+
+        * instance of :class:`Module` (instance of it's subclass counts)
+
+          ::
+
+            class MyModule(Module):
+                def configure(self, binder):
+                    binder.bind(str, to='s')
+
+            binder.install(MyModule())
+
+        * subclass of :class:`Module` - the subclass needs to be instantiable so if it
+          expects any parameters they need to be injected
+
+          ::
+
+            binder.install(MyModule)
         """
+        if hasattr(module, '__bindings__') or \
+                hasattr(module, 'configure') and hasattr(module.configure, '__bindings__'):
+            warnings.warn(
+                'Injector Modules (ie. %s) should not be injected. This can result in non-deterministic '
+                'initialization. Support will be removed in the next release of Injector.' %
+                module.__name__ if hasattr(module, '__name__') else module.__class__.__name__,
+                RuntimeWarning,
+                stacklevel=3,
+                )
         if type(module) is type and issubclass(module, Module):
             instance = self.injector.create_object(module)
             instance(self)
@@ -311,32 +375,36 @@ class Binder(object):
                 args=(self,),
             )
 
-    def create_binding(self, interface, to=None, annotation=None, scope=None):
+    def create_binding(self, interface, to=None, scope=None):
         provider = self.provider_for(interface, to)
         scope = scope or getattr(to or interface, '__scope__', NoScope)
         if isinstance(scope, ScopeDecorator):
             scope = scope.scope
-        return Binding(interface, annotation, provider, scope)
+        return Binding(interface, provider, scope)
 
     def provider_for(self, interface, to=None):
         if isinstance(to, Provider):
             return to
+        elif isinstance(interface, Provider):
+            return interface
         elif isinstance(to, (types.FunctionType, types.LambdaType,
                              types.MethodType, types.BuiltinFunctionType,
                              types.BuiltinMethodType)):
             return CallableProvider(to)
         elif issubclass(type(to), type):
             return ClassProvider(to, self.injector)
-        elif isinstance(interface, ParameterizedBuilder):
-            builder = AssistedBuilderImplementation(self.injector, interface.interface, None, None)
-            return CallableProvider(lambda: builder.build(**interface.kwargs))
+        elif isinstance(interface, BoundKey):
+            @inject(**interface.kwargs)
+            def proxy(**kwargs):
+                return interface.interface(**kwargs)
+            return CallableProvider(lambda: self.injector.call_with_injection(proxy))
         elif isinstance(interface, AssistedBuilder):
             builder = AssistedBuilderImplementation(self.injector, *interface)
             return InstanceProvider(builder)
         elif isinstance(interface, (tuple, type)) and isinstance(to, interface):
             return InstanceProvider(to)
         elif issubclass(type(interface), type) or isinstance(interface, (tuple, list)):
-            if issubclass(interface, (BaseKey, BaseMappingKey, BaseSequenceKey)):
+            if issubclass(interface, (BaseKey, BaseMappingKey, BaseSequenceKey)) and to is not None:
                 return InstanceProvider(to)
             return ClassProvider(interface, self.injector)
         elif hasattr(interface, '__call__'):
@@ -365,10 +433,7 @@ class Binder(object):
             return self._get_binding(key)
         except (KeyError, UnsatisfiedRequirement):
             if self._auto_bind:
-                binding = self.create_binding(
-                    key.interface,
-                    annotation=key.annotation,
-                    )
+                binding = self.create_binding(key.interface)
                 self._bindings[key] = binding
                 return binding
             raise UnsatisfiedRequirement(cls, key)
@@ -409,11 +474,9 @@ class ScopeDecorator(object):
         cls.__scope__ = self.scope
         binding = getattr(cls, '__binding__', None)
         if binding:
-            new_binding = Binding(
-                interface=binding.interface,
-                annotation=binding.annotation,
-                provider=binding.provider,
-                scope=self.scope)
+            new_binding = Binding(interface=binding.interface,
+                                  provider=binding.provider,
+                                  scope=self.scope)
             setattr(cls, '__binding__', new_binding)
         return cls
 
@@ -493,9 +556,8 @@ class Module(object):
                 binder.bind(
                     binding.interface,
                     to=types.MethodType(binding.provider, self),
-                    annotation=binding.annotation,
                     scope=binding.scope,
-                    )
+                )
         self.configure(binder)
 
     def configure(self, binder):
@@ -503,23 +565,39 @@ class Module(object):
 
 
 class Injector(object):
-    """Initialise and use an object dependency graph."""
+    """
+    :param modules: Optional - a configuration module or iterable of configuration modules.
+        Each module will be installed in current :class:`Binder` using :meth:`Binder.install`.
+
+        Consult :meth:`Binder.install` documentation for the details.
+
+    :param auto_bind: Whether to automatically bind missing types.
+    :param parent: Parent injector.
+    :param use_annotations: Attempt to infer injected arguments using Python3 argument annotations.
+
+    If you use Python 3 you can make Injector use constructor parameter annotations to
+    determine class dependencies. The following code::
+
+        class B(object):
+            @inject(a=A):
+            def __init__(self, a):
+                self.a = a
+
+    can now be written as::
+
+        class B(object):
+            def __init__(self, a:A):
+                self.a = a
+
+    To enable Python 3 annotation support, instantiate your :class:`Injector` with
+    ``use_annotations=True``.
+
+
+    .. versionadded:: 0.7.5
+        ``use_annotations`` parameter
+    """
 
     def __init__(self, modules=None, auto_bind=True, parent=None, use_annotations=False):
-        """Construct a new Injector.
-
-        :param modules: A callable, class, or list of callables/classes, used to configure the
-                        Binder associated with this Injector. Typically these
-                        callables will be subclasses of :class:`Module`.
-
-                        In case of class, it's instance will be created using parameterless
-                        constructor before the configuration process begins.
-
-                        Signature is ``configure(binder)``.
-        :param auto_bind: Whether to automatically bind missing types.
-        :param parent: Parent injector.
-        :param use_annotations: Attempt to infer injected arguments using Python3 argument annotations.
-        """
         # Stack of keys currently being injected. Used to detect circular
         # dependencies.
         self._stack = ()
@@ -552,21 +630,20 @@ class Injector(object):
     def _log_prefix(self):
         return '>' * (len(self._stack) + 1) + ' '
 
-    def get(self, interface, annotation=None, scope=None):
+    def get(self, interface, scope=None):
         """Get an instance of the given interface.
 
         :param interface: Interface whose implementation we want.
-        :param annotation: Optional annotation of the specific implementation.
         :param scope: Class of the Scope in which to resolve.
         :returns: An implementation of interface.
         """
-        key = BindingKey(interface, annotation)
+        key = BindingKey(interface)
         binding = self.binder.get_binding(None, key)
         scope = scope or binding.scope
         if isinstance(scope, ScopeDecorator):
             scope = scope.scope
         # Fetch the corresponding Scope instance from the Binder.
-        scope_key = BindingKey(scope, None)
+        scope_key = BindingKey(scope)
         try:
             scope_binding = self.binder.get_binding(None, scope_key)
             scope_instance = scope_binding.provider.get()
@@ -574,7 +651,8 @@ class Injector(object):
             raise Error('%s; scopes must be explicitly bound '
                         'with Binder.bind_scope(scope_cls)' % e)
 
-        log.debug('%sInjector.get(%r, annotation=%r, scope=%r) using %r', self._log_prefix, interface, annotation, scope, binding.provider)
+        log.debug('%sInjector.get(%r, scope=%r) using %r',
+                  self._log_prefix, interface, scope, binding.provider)
         result = scope_instance.get(key, binding.provider).get()
         log.debug('%s -> %r', self._log_prefix, result)
         return result
@@ -587,7 +665,11 @@ class Injector(object):
         additional_kwargs = additional_kwargs or {}
         log.debug('%sCreating %r object with %r', self._log_prefix, cls, additional_kwargs)
 
-        if self.use_annotations and hasattr(cls, '__init__') and not hasattr(cls.__init__, '__binding__'):
+        # (cls.__init__ is not object.__init__) is a workaround for Python 3.3
+        # where object.__init__ is a slot wrapper that can't be inspected
+        if self.use_annotations and hasattr(cls, '__init__') and \
+           not hasattr(cls.__init__, '__binding__') and \
+           cls.__init__ is not object.__init__:
             bindings = self._infer_injected_bindings(cls.__init__)
             cls.__init__ = inject(**bindings)(cls.__init__)
 
@@ -619,8 +701,42 @@ class Injector(object):
         return dict(spec.annotations.items())
 
     def install_into(self, instance):
-        """
-        Puts injector reference in given object.
+        """Put injector reference in given object.
+
+        This method has, in general, two applications:
+
+        * Injector internal use (not documented here)
+        * Making it possible to inject into methods of an object that wasn't created
+            using Injector. Usually it's because you either don't control the instantiation
+            process, it'd generate unnecessary boilerplate or it's just easier this way.
+
+            For example, in application main script::
+
+                from injector import Injector
+
+                class Main(object):
+                    def __init__(self):
+                        def configure(binder):
+                            binder.bind(str, to='Hello!')
+
+                        injector = Injector(configure)
+                        injector.install_into(self)
+
+                    @inject(s=str)
+                    def run(self, s):
+                        print(s)
+
+                if __name__ == '__main__':
+                    main = Main()
+                    main.run()
+
+
+        .. note:: You don't need to use this method if the object is created using `Injector`.
+
+        .. warning:: Using `install_into` to install :class:`Injector` reference into an object
+            created by different :class:`Injector` instance may very likely result in unexpected
+            behaviour of that object immediately or in the future.
+
         """
         instance.__injector__ = self
 
@@ -696,18 +812,14 @@ class Injector(object):
         if key in self._stack:
             raise CircularDependency(
                 'circular dependency detected: %s -> %s' %
-                (' -> '.join(map(repr_key, self._stack)),
-                repr_key(key)),
-                )
+                (' -> '.join(map(repr_key, self._stack)), repr_key(key))
+            )
 
         self._stack += (key,)
         try:
             for arg, key in bindings.items():
                 try:
-                    instance = self.get(
-                        key.interface,
-                        annotation=key.annotation,
-                        )
+                    instance = self.get(key.interface)
                 except UnsatisfiedRequirement as e:
                     if not e.args[0]:
                         e = UnsatisfiedRequirement(owner_key, e.args[1])
@@ -723,7 +835,7 @@ def with_injector(*injector_args, **injector_kwargs):
     """Decorator for a method. Installs Injector object which the method
     belongs to before the decorated method is executed.
 
-    Parameters are the same as for Injector constructor.
+    Parameters are the same as for :class:`Injector` constructor.
     """
     def wrapper(f):
         @functools.wraps(f)
@@ -737,21 +849,20 @@ def with_injector(*injector_args, **injector_kwargs):
     return wrapper
 
 
-def provides(interface, annotation=None, scope=None, eager=False):
+def provides(interface, scope=None, eager=False):
     """Decorator for :class:`Module` methods, registering a provider of a type.
 
     >>> class MyModule(Module):
-    ...   @provides(str, annotation='annotation')
+    ...   @provides(str)
     ...   def provide_name(self):
-    ...     return 'Bob'
+    ...       return 'Bob'
 
     :param interface: Interface to provide.
-    :param annotation: Optional annotation value.
     :param scope: Optional scope of provided value.
     """
     def wrapper(provider):
         scope_ = scope or getattr(provider, '__scope__', getattr(wrapper, '__scope__', None))
-        provider.__binding__ = Binding(interface, annotation, provider, scope_)
+        provider.__binding__ = Binding(interface, provider, scope_)
         return provider
 
     return wrapper
@@ -809,7 +920,7 @@ def inject(**bindings):
 
     def method_wrapper(f):
         for key, value in bindings.items():
-            bindings[key] = BindingKey(value, None)
+            bindings[key] = BindingKey(value)
         argspec = getargspec(f)
         if argspec.args and argspec.args[0] == 'self':
             @functools.wraps(f)
@@ -870,21 +981,7 @@ def inject(**bindings):
     return multi_wrapper
 
 
-class BaseAnnotation(object):
-    """Annotation base type."""
-
-
-def Annotation(name):
-    """Create a new annotation type.
-
-    Useful for declaring a unique annotation type.
-
-    :param name: Name of the annotation type.
-    :returns: Newly created unique type.
-    """
-    return type(name, (BaseAnnotation,), {})
-
-
+@private
 class BaseKey(object):
     """Base type for binding keys."""
 
@@ -896,17 +993,6 @@ class BaseKey(object):
 def Key(name):
     """Create a new type key.
 
-    Typically when using Injector, complex types can be bound to providers
-    directly.
-
-    Keys are a convenient alternative to binding to (type, annotation) pairs,
-    particularly when non-unique types such as str or int are being bound.
-
-    eg. if using @provides(str), chances of collision are almost guaranteed.
-    One solution is to use @provides(str, annotation='unique') everywhere
-    you wish to inject the value, but this is verbose and error prone. Keys
-    solve this problem:
-
     >>> Age = Key('Age')
     >>> def configure(binder):
     ...   binder.bind(Age, to=90)
@@ -921,6 +1007,7 @@ def Key(name):
     return type(name, (BaseKey,), {})
 
 
+@private
 class BaseMappingKey(dict):
     """Base type for mapping binding keys."""
     def __init__(self):
@@ -938,6 +1025,7 @@ def MappingKey(name):
     return type(name, (BaseMappingKey,), {})
 
 
+@private
 class BaseSequenceKey(list):
     """Base type for mapping sequence keys."""
     def __init__(self):
@@ -955,10 +1043,23 @@ def SequenceKey(name):
     return type(name, (BaseSequenceKey,), {})
 
 
-class ParameterizedBuilder(tuple):
+class BoundKey(tuple):
+    """A BoundKey provides a key to a type with pre-injected arguments.
+
+    >>> class A(object):
+    ...   def __init__(self, a, b):
+    ...     self.a = a
+    ...     self.b = b
+    >>> InjectedA = BoundKey(A, a=InstanceProvider(1), b=InstanceProvider(2))
+    >>> injector = Injector()
+    >>> a = injector.get(InjectedA)
+    >>> a.a, a.b
+    (1, 2)
+    """
+
     def __new__(cls, interface, **kwargs):
         kwargs = tuple(sorted(kwargs.items()))
-        return super(ParameterizedBuilder, cls).__new__(cls, (interface, kwargs))
+        return super(BoundKey, cls).__new__(cls, (interface, kwargs))
 
     @property
     def interface(self):
@@ -994,7 +1095,7 @@ class AssistedBuilderImplementation(namedtuple(
         return self.injector.create_object(cls, additional_kwargs=kwargs)
 
     def build_interface(self, **kwargs):
-        key = BindingKey(self.interface, None)
+        key = BindingKey(self.interface)
         binder = self.injector.binder
         binding = binder.get_binding(None, key)
         provider = binding.provider
